@@ -104,6 +104,7 @@ struct CacheContinuation:public Continuation
   ClusterMachine *past_probes[CONFIGURATION_HISTORY_PROBE_DEPTH];
   ink_hrtime start_time;
   ClusterMachine *from;
+  ClusterHandler *ch;
   VConnection *cache_vc;
   bool cache_read;
   int result;                   // return event code
@@ -214,7 +215,7 @@ struct CacheContinuation:public Continuation
     if (rw_buf_msg_len <= DEFAULT_MAX_BUFFER_SIZE) {
       rw_buf_msg = new_IOBufferData(buffer_size_to_index(rw_buf_msg_len, MAX_BUFFER_SIZE_INDEX));
     } else {
-      rw_buf_msg = new_xmalloc_IOBufferData(xmalloc(rw_buf_msg_len), rw_buf_msg_len);
+      rw_buf_msg = new_xmalloc_IOBufferData(ats_malloc(rw_buf_msg_len), rw_buf_msg_len);
     }
   }
 
@@ -427,8 +428,8 @@ struct CacheLookupMsg:public ClusterMessageHeader
   {
     if (NeedByteSwap()) {
       ink_release_assert(!"No byte swap for INK_MD5");
-      swap32(&seq_number);
-      swap32(&frag_type);
+      ats_swap32(&seq_number);
+      ats_swap32(&frag_type);
     }
   }
   //////////////////////////////////////////////////////////////////////////
@@ -479,14 +480,14 @@ struct CacheOpMsg_long:public ClusterMessageHeader
   {
     if (NeedByteSwap()) {
       ink_release_assert(!"No byte swap for INK_MD5");
-      swap16(&cfl_flags);
-      swap32(&seq_number);
-      swap32(&nbytes);
-      swap32(&data);
-      swap32((uint32_t *) & channel);
+      ats_swap16(&cfl_flags);
+      ats_swap32(&seq_number);
+      ats_swap32(&nbytes);
+      ats_swap32(&data);
+      ats_swap32((uint32_t *) & channel);
       token.SwapBytes();
-      swap32((uint32_t *) & buffer_size);
-      swap32((uint32_t *) & frag_type);
+      ats_swap32((uint32_t *) & buffer_size);
+      ats_swap32((uint32_t *) & frag_type);
     }
   }
   //////////////////////////////////////////////////////////////////////////
@@ -538,13 +539,13 @@ struct CacheOpMsg_short:public ClusterMessageHeader
   {
     if (NeedByteSwap()) {
       ink_release_assert(!"No byte swap for INK_MD5");
-      swap16(&cfl_flags);
-      swap32(&seq_number);
-      swap32(&nbytes);
-      swap32(&data);
+      ats_swap16(&cfl_flags);
+      ats_swap32(&seq_number);
+      ats_swap32(&nbytes);
+      ats_swap32(&data);
       if (opcode == CACHE_OPEN_READ) {
-        swap32((uint32_t *) & buffer_size);
-        swap32((uint32_t *) & channel);
+        ats_swap32((uint32_t *) & buffer_size);
+        ats_swap32((uint32_t *) & channel);
         token.SwapBytes();
       }
     }
@@ -591,8 +592,8 @@ struct CacheOpMsg_short_2:public ClusterMessageHeader
     if (NeedByteSwap()) {
       ink_release_assert(!"No byte swap for MD5_1");
       ink_release_assert(!"No byte swap for MD5_2");
-      swap16(&cfl_flags);
-      swap32(&seq_number);
+      ats_swap16(&cfl_flags);
+      ats_swap32(&seq_number);
     }
   }
   //////////////////////////////////////////////////////////////////////////
@@ -633,8 +634,8 @@ struct CacheOpReplyMsg:public ClusterMessageHeader
   inline void SwapBytes()
   {
     if (NeedByteSwap()) {
-      swap32(&seq_number);
-      swap32((uint32_t *) & result);
+      ats_swap32(&seq_number);
+      ats_swap32((uint32_t *) & result);
       token.SwapBytes();
     }
   }
