@@ -34,7 +34,6 @@
 
 #include <stdio.h>
 
-#ifndef _WIN32
 #ifdef __cplusplus
 extern "C"
 {
@@ -47,6 +46,4 @@ void *event_poll_thread_main(void *arg);
 }
 #endif /* __cplusplus */
 
-#endif /* !_WIN32 */
-
 #endif /* _EVENT_REGISTRATION_H */
