@@ -35,12 +35,26 @@
 #include "StatSystem.h"
 #include "P_EventSystem.h"
 #include "P_Cache.h"
+#include "hdrs/HdrToken.h"
+
+#include <sstream>
 
 #define IPAllowRegisterConfigUpdateFunc REC_RegisterConfigUpdateFunc
 #define IPAllowReadConfigStringAlloc REC_ReadConfigStringAlloc
 
-IpAllow *ip_allow_table = NULL;
-Ptr<ProxyMutex> ip_reconfig_mutex;
+enum AclOp {
+  ACL_OP_ALLOW, ///< Allow access.
+  ACL_OP_DENY, ///< Deny access.
+};
+
+IpAllow* IpAllow::_instance = NULL;
+
+// Mask for all methods.
+// This can't be computed properly at process start, so it's delayed
+// until the instance is initialized.
+uint32_t IpAllow::ALL_METHOD_MASK;
+
+static Ptr<ProxyMutex> ip_reconfig_mutex;
 
 //
 // struct IPAllow_FreerContinuation
@@ -59,7 +73,7 @@ struct IPAllow_FreerContinuation: public Continuation
     Debug("ip-allow", "Deleting old table");
     delete p;
     delete this;
-      return EVENT_DONE;
+    return EVENT_DONE;
   }
   IPAllow_FreerContinuation(IpAllow * ap):Continuation(NULL), p(ap)
   {
@@ -78,7 +92,7 @@ struct IPAllow_UpdateContinuation: public Continuation
   {
     NOWARN_UNUSED(etype);
     NOWARN_UNUSED(data);
-    reloadIPAllow();
+    IpAllow::ReloadInstance();
     delete this;
       return EVENT_DONE;
   }
@@ -103,34 +117,33 @@ ipAllowFile_CB(const char *name, RecDataT data_type, RecData data, void *cookie)
 //   Begin API functions
 //
 void
-initIPAllow()
-{
-
+IpAllow::InitInstance() {
   // Should not have been initialized before
-  ink_assert(ip_allow_table == NULL);
+  ink_assert(_instance == NULL);
+
+  ALL_METHOD_MASK = ~(~0 << HTTP_WKSIDX_METHODS_CNT);
 
   ip_reconfig_mutex = new_ProxyMutex();
 
-  ip_allow_table = NEW(new IpAllow("proxy.config.cache.ip_allow.filename", "IpAllow", "ip_allow"));
-  ip_allow_table->BuildTable();
+  _instance = NEW(new self("proxy.config.cache.ip_allow.filename", "IpAllow", "ip_allow"));
+  _instance->BuildTable();
 
   IPAllowRegisterConfigUpdateFunc("proxy.config.cache.ip_allow.filename", ipAllowFile_CB, NULL);
 }
 
 void
-reloadIPAllow()
-{
-  IpAllow *new_table;
+IpAllow::ReloadInstance() {
+  self *new_table;
 
   Debug("ip_allow", "ip_allow.config updated, reloading");
 
   // Schedule the current table for deallocation in the future
-  eventProcessor.schedule_in(NEW(new IPAllow_FreerContinuation(ip_allow_table)), IP_ALLOW_TIMEOUT, ET_CACHE);
+  eventProcessor.schedule_in(NEW(new IPAllow_FreerContinuation(_instance)), IP_ALLOW_TIMEOUT, ET_CACHE);
 
-  new_table = NEW(new IpAllow("proxy.config.cache.ip_allow.filename", "IpAllow", "ip_allow"));
+  new_table = NEW(new self("proxy.config.cache.ip_allow.filename", "IpAllow", "ip_allow"));
   new_table->BuildTable();
 
-  ink_atomic_swap_ptr(&ip_allow_table, new_table);
+  ink_atomic_swap_ptr(_instance, new_table);
 }
 
 //
@@ -138,23 +151,24 @@ reloadIPAllow()
 //
 
 
-IpAllow::IpAllow(const char *config_var, const char *name, const char *action_val):
-IpLookup(name),
-config_file_var(config_var),
-module_name(name),
-action(action_val),
-err_allow_all(false)
+IpAllow::IpAllow(
+  const char *config_var,
+  const char *name,
+  const char *action_val
+) : config_file_var(config_var),
+    module_name(name),
+    action(action_val)
 {
 
   char *config_file;
 
-  config_file_var = xstrdup(config_var);
+  config_file_var = ats_strdup(config_var);
   config_file_path[0] = '\0';
 
   IPAllowReadConfigStringAlloc(config_file, (char *) config_file_var);
   ink_release_assert(config_file != NULL);
   ink_filepath_make(config_file_path, sizeof(config_file_path), system_config_directory, config_file);
-  xfree(config_file);
+  ats_free(config_file);
 }
 
 IpAllow::~IpAllow()
@@ -162,13 +176,43 @@ IpAllow::~IpAllow()
 }
 
 void
-IpAllow::Print()
-{
-  printf("IpAllow Table with %d entries\n", num_el);
-  if (err_allow_all == true) {
-    printf("\t err_allow_all is true\n");
+IpAllow::Print() {
+  std::ostringstream s;
+  s << _map.getCount() << " ACL entries";
+  s << '.';
+  for ( IpMap::iterator spot(_map.begin()), limit(_map.end())
+      ; spot != limit
+      ; ++spot
+  ) {
+    char text[INET6_ADDRSTRLEN];
+    AclRecord const* ar = static_cast<AclRecord const*>(spot->data());
+
+    s << std::endl << "  Line " << ar->_src_line << ": "
+      << ats_ip_ntop(spot->min(), text, sizeof text)
+      ;
+    if (0 != ats_ip_addr_cmp(spot->min(), spot->max())) {
+      s << " - " << ats_ip_ntop(spot->max(), text, sizeof text);
+    }
+    s << " method=";
+    uint32_t mask = ALL_METHOD_MASK & ar->_method_mask;
+    if (ALL_METHOD_MASK == mask) {
+      s << "ALL";
+    } else if (0 == mask) {
+      s << "NONE";
+    } else {
+      bool leader = false; // need leading vbar?
+      uint32_t test_mask = 1; // mask for current method.
+      for ( int i = 0 ; i < HTTP_WKSIDX_METHODS_CNT ; ++i, test_mask<<=1 ) {
+        if (mask & test_mask) {
+          if (leader)
+            s << '|';
+          s << hdrtoken_index_to_wks(i + HTTP_WKSIDX_CONNECT);
+          leader = true;
+        }
+      }
+    }
   }
-  IpLookup::Print();
+  Debug("ip-allow", "%s", s.str().c_str());
 }
 
 int
@@ -180,18 +224,17 @@ IpAllow::BuildTable()
   char errBuf[1024];
   char *file_buf = NULL;
   int line_num = 0;
-  ip_addr_t addr1 = 0;
-  ip_addr_t addr2 = 0;
+  IpEndpoint addr1;
+  IpEndpoint addr2;
   matcher_line line_info;
   bool alarmAlready = false;
 
   // Table should be empty
-  ink_assert(num_el == 0);
+  ink_assert(_map.getCount() == 0);
 
   file_buf = readIntoBuffer(config_file_path, module_name, NULL);
 
   if (file_buf == NULL) {
-    err_allow_all = false;
     Warning("%s Failed to read %s. All IP Addresses will be blocked", module_name, config_file_path);
     return 1;
   }
@@ -199,7 +242,7 @@ IpAllow::BuildTable()
   line = tokLine(file_buf, &tok_state);
   while (line != NULL) {
 
-    line_num++;
+    ++line_num;
 
     // skip all blank spaces at beginning of line
     while (*line && isspace(*line)) {
@@ -218,7 +261,7 @@ IpAllow::BuildTable()
 
         ink_assert(line_info.type == MATCH_IP);
 
-        errPtr = ExtractIpRange(line_info.line[1][line_info.dest_entry], &addr1, &addr2);
+        errPtr = ExtractIpRange(line_info.line[1][line_info.dest_entry], &addr1.sa, &addr2.sa);
 
         if (errPtr != NULL) {
           snprintf(errBuf, sizeof(errBuf), "%s discarding %s entry at line %d : %s",
@@ -226,32 +269,75 @@ IpAllow::BuildTable()
           SignalError(errBuf, alarmAlready);
         } else {
           // INKqa05845
-          // Search for "action=ip_allow" or "action=ip_deny".
+          // Search for "action=ip_allow method=PURGE method=GET ..." or "action=ip_deny method=PURGE method=GET ...".
           char *label, *val;
-          IpAllowRecord *rec;
+          uint32_t acl_method_mask = 0;
+          AclOp op = ACL_OP_DENY; // "shut up", I explained to the compiler.
+          bool op_found = false, method_found = false;
           for (int i = 0; i < MATCHER_MAX_TOKENS; i++) {
             label = line_info.line[0][i];
             val = line_info.line[1][i];
-            if (label == NULL)
+            if (label == NULL) {
               continue;
+            }
             if (strcasecmp(label, "action") == 0) {
               if (strcasecmp(val, "ip_allow") == 0) {
-                rec = (IpAllowRecord *) xmalloc(sizeof(IpAllowRecord));
-                rec->access = IP_ALLOW;
-                rec->line_num = line_num;
-                this->NewEntry(addr1, addr2, (void *) rec);
+                op_found = true, op = ACL_OP_ALLOW;
               } else if (strcasecmp(val, "ip_deny") == 0) {
-                rec = (IpAllowRecord *) xmalloc(sizeof(IpAllowRecord));
-                rec->access = IP_DENY;
-                rec->line_num = line_num;
-                this->NewEntry(addr1, addr2, (void *) rec);
-              } else {
-                snprintf(errBuf, sizeof(errBuf), "%s discarding %s entry at line %d : %s", module_name, config_file_path, line_num, "Invalid action specified");        //changed by YTS Team, yamsat bug id -59022
-                SignalError(errBuf, alarmAlready);
+                op_found = true, op = ACL_OP_DENY;
               }
             }
           }
-          // this->NewEntry(addr1, addr2, NULL);
+          if (op_found) {
+            // Loop again for methods, (in case action= appears after method=)
+            for (int i = 0; i < MATCHER_MAX_TOKENS; i++) {
+              label = line_info.line[0][i];
+              val = line_info.line[1][i];
+              if (label == NULL) {
+                continue;
+              }
+              if (strcasecmp(label, "method") == 0) {
+                char *method_name, *sep_ptr = 0;
+                // Parse method="GET|HEAD"
+                for (method_name = strtok_r(val, "|", &sep_ptr); method_name != NULL; method_name = strtok_r(NULL, "|", &sep_ptr)) {
+                  if (strcasecmp(method_name, "ALL") == 0) {
+                    method_found = false;  // in case someone does method=GET|ALL
+                    break;
+                  } else {
+                    int method_idx = hdrtoken_tokenize(method_name, strlen(method_name));
+                    if (method_idx < HTTP_WKSIDX_CONNECT || method_idx >= HTTP_WKSIDX_CONNECT + HTTP_WKSIDX_METHODS_CNT) {
+                      Warning("Method name '%s' on line %d is not valid. Ignoring.", method_name, line_num);
+                    } else { // valid method.
+                      method_found = true;
+                      acl_method_mask |= MethodIdxToMask(method_idx);
+                    }
+                  }
+                }
+              }
+            }
+            // If method not specified, default to ALL
+            if (!method_found) {
+              method_found = true, acl_method_mask = ALL_METHOD_MASK;
+            }
+            // When deny, use bitwise complement.  (Make the rule 'allow for all
+            // methods except those specified')
+            if (op == ACL_OP_DENY) {
+              acl_method_mask = ALL_METHOD_MASK & ~acl_method_mask;
+            }
+          }
+
+          if (method_found) {
+            _acls.push_back(AclRecord(acl_method_mask, line_num));
+            // Color with index because at this point the address
+            // is volatile.
+            _map.fill(
+              &addr1, &addr2,
+              reinterpret_cast<void*>(_acls.length()-1)
+            );
+          } else {
+            snprintf(errBuf, sizeof(errBuf), "%s discarding %s entry at line %d : %s", module_name, config_file_path, line_num, "Invalid action/method specified");        //changed by YTS Team, yamsat bug id -59022
+            SignalError(errBuf, alarmAlready);
+          }
         }
       }
     }
@@ -259,15 +345,22 @@ IpAllow::BuildTable()
     line = tokLine(NULL, &tok_state);
   }
 
-  if (num_el == 0) {
+  if (_map.getCount() == 0) {
     Warning("%s No entries in %s. All IP Addresses will be blocked", module_name, config_file_path);
-    err_allow_all = false;
+  } else {
+    // convert the coloring from indices to pointers.
+    for ( IpMap::iterator spot(_map.begin()), limit(_map.end())
+        ; spot != limit
+        ; ++spot
+    ) {
+      spot->setData(&_acls[reinterpret_cast<size_t>(spot->data())]);
+    }
   }
 
   if (is_debug_tag_set("ip-allow")) {
     Print();
   }
 
-  xfree(file_buf);
+  ats_free(file_buf);
   return 0;
 }
