@@ -170,6 +170,31 @@ Log::add_to_inactive(LogObject * object)
   thread will keep track of executions per period.
   -------------------------------------------------------------------------*/
 
+/*-------------------------------------------------------------------------
+  PeriodicWakeup
+
+  This continuation is invoked each second to wake-up the flush thread,
+  just in case it's sleeping on the job.
+  -------------------------------------------------------------------------*/
+
+struct PeriodicWakeup;
+typedef int (PeriodicWakeup::*PeriodicWakeupHandler)(int, void *);
+struct PeriodicWakeup : Continuation {
+
+    int wakeup (int event, Event *e)
+    {
+        NOWARN_UNUSED (event); NOWARN_UNUSED (e);
+        ink_cond_signal (&Log::flush_cond);
+        return EVENT_CONT;
+    }
+
+    PeriodicWakeup () : Continuation (new_ProxyMutex())
+    {
+        SET_HANDLER ((PeriodicWakeupHandler)&PeriodicWakeup::wakeup);
+    }
+};
+
+
 /*-------------------------------------------------------------------------
   Log::periodic_tasks
 
@@ -249,12 +274,12 @@ Log::periodic_tasks(long time_now)
       Log::config->roll_log_files_now = FALSE;
     } else {
       if (error_log) {
-        num_rolled += error_log->roll_files_if_needed(time_now);
+        num_rolled += error_log->roll_files(time_now);
       }
       if (global_scrap_object) {
-        num_rolled += global_scrap_object->roll_files_if_needed(time_now);
+        num_rolled += global_scrap_object->roll_files(time_now);
       }
-      num_rolled += Log::config->log_object_manager.roll_files_if_needed(time_now);
+      num_rolled += Log::config->log_object_manager.roll_files(time_now);
     }
 
   }
@@ -321,19 +346,17 @@ Log::init_fields()
   field_symbol_hash = ink_hash_table_create(InkHashTableKeyType_String);
 
   // client -> proxy fields
-  Ptr<LogFieldAliasIP> ip_map = NEW(new LogFieldAliasIP);
   field = NEW(new LogField("client_host_ip", "chi",
-                           LogField::STRING,
+                           LogField::IP,
                            &LogAccess::marshal_client_host_ip,
-                           &LogAccess::unmarshal_str));
+                           &LogAccess::unmarshal_ip_to_str));
   global_field_list.add(field, false);
   ink_hash_table_insert(field_symbol_hash, "chi", field);
 
-  Ptr<LogFieldAliasIPhex> ip_map_hex = NEW(new LogFieldAliasIPhex);
   field = NEW(new LogField("client_host_ip_hex", "chih",
-                           LogField::sINT,
+                           LogField::IP,
                            &LogAccess::marshal_client_host_ip,
-                           &LogAccess::unmarshal_ip, (Ptr<LogFieldAliasMap>) ip_map_hex));
+                           &LogAccess::unmarshal_ip_to_hex));
   global_field_list.add(field, false);
   ink_hash_table_insert(field_symbol_hash, "chih", field);
 
@@ -428,6 +451,13 @@ Log::init_fields()
   global_field_list.add(field, false);
   ink_hash_table_insert(field_symbol_hash, "cquup", field);
 
+  field = NEW(new LogField("client_req_unmapped_url_host", "cquuh",
+                           LogField::STRING,
+                           &LogAccess::marshal_client_req_unmapped_url_host,
+                           &LogAccess::unmarshal_str));
+  global_field_list.add(field, false);
+  ink_hash_table_insert(field_symbol_hash, "cquuh", field);
+
   field = NEW(new LogField("client_req_url_scheme", "cqus",
                            LogField::STRING,
                            &LogAccess::marshal_client_req_url_scheme,
@@ -542,12 +572,12 @@ Log::init_fields()
                        SQUID_LOG_TCP_MISS, "TCP_MISS",
                        SQUID_LOG_TCP_EXPIRED_MISS, "TCP_EXPIRED_MISS",
                        SQUID_LOG_TCP_REFRESH_HIT, "TCP_REFRESH_HIT",
-                       SQUID_LOG_TCP_REF_FAIL_HIT, "TCP_REF_FAIL_HIT",
+                       SQUID_LOG_TCP_REF_FAIL_HIT, "TCP_REFRESH_FAIL_HIT",
                        SQUID_LOG_TCP_REFRESH_MISS, "TCP_REFRESH_MISS",
-                       SQUID_LOG_TCP_CLIENT_REFRESH, "TCP_CLIENT_REFRESH",
+                       SQUID_LOG_TCP_CLIENT_REFRESH, "TCP_CLIENT_REFRESH_MISS",
                        SQUID_LOG_TCP_IMS_HIT, "TCP_IMS_HIT",
                        SQUID_LOG_TCP_IMS_MISS, "TCP_IMS_MISS",
-                       SQUID_LOG_TCP_SWAPFAIL, "TCP_SWAPFAIL",
+                       SQUID_LOG_TCP_SWAPFAIL, "TCP_SWAPFAIL_MISS",
                        SQUID_LOG_TCP_DENIED, "TCP_DENIED",
                        SQUID_LOG_TCP_WEBFETCH_MISS, "TCP_WEBFETCH_MISS",
                        SQUID_LOG_TCP_SPIDER_BYPASS, "TCP_SPIDER_BYPASS",
@@ -645,9 +675,9 @@ Log::init_fields()
   ink_hash_table_insert(field_symbol_hash, "pqsn", field);
 
   field = NEW(new LogField("proxy_req_server_ip", "pqsi",
-                           LogField::sINT,
+                           LogField::IP,
                            &LogAccess::marshal_proxy_req_server_ip,
-                           &LogAccess::unmarshal_ip, (Ptr<LogFieldAliasMap>) ip_map));
+      &LogAccess::unmarshal_ip_to_str));
   global_field_list.add(field, false);
   ink_hash_table_insert(field_symbol_hash, "pqsi", field);
 
@@ -705,9 +735,9 @@ Log::init_fields()
   ink_hash_table_insert(field_symbol_hash, "phn", field);
 
   field = NEW(new LogField("proxy_host_ip", "phi",
-                           LogField::STRING,
+                           LogField::IP,
                            &LogAccess::marshal_proxy_host_ip,
-                           &LogAccess::unmarshal_str));
+                           &LogAccess::unmarshal_ip_to_str));
   global_field_list.add(field, false);
   ink_hash_table_insert(field_symbol_hash, "phi", field);
 
@@ -723,9 +753,10 @@ Log::init_fields()
   // server -> proxy fields
 
   field = NEW(new LogField("server_host_ip", "shi",
-                           LogField::sINT,
+                           LogField::IP,
                            &LogAccess::marshal_server_host_ip,
-                           &LogAccess::unmarshal_ip, (Ptr<LogFieldAliasMap>) ip_map));
+      &LogAccess::unmarshal_ip_to_str));
+
   global_field_list.add(field, false);
   ink_hash_table_insert(field_symbol_hash, "shi", field);
 
@@ -758,6 +789,30 @@ Log::init_fields()
   global_field_list.add(field, false);
   ink_hash_table_insert(field_symbol_hash, "sshv", field);
 
+  field = NEW(new LogField("cached_resp_status_code", "csssc",
+                           LogField::sINT,
+                           &LogAccess::marshal_cache_resp_status_code, &LogAccess::unmarshal_http_status));
+  global_field_list.add(field, false);
+  ink_hash_table_insert(field_symbol_hash, "csssc", field);
+
+  field = NEW(new LogField("cached_resp_content_len", "csscl",
+                           LogField::sINT,
+                           &LogAccess::marshal_cache_resp_content_len, &LogAccess::unmarshal_int_to_str));
+  global_field_list.add(field, false);
+  ink_hash_table_insert(field_symbol_hash, "csscl", field);
+
+  field = NEW(new LogField("cached_resp_header_len", "csshl",
+                           LogField::sINT,
+                           &LogAccess::marshal_cache_resp_header_len, &LogAccess::unmarshal_int_to_str));
+  global_field_list.add(field, false);
+  ink_hash_table_insert(field_symbol_hash, "csshl", field);
+
+  field = NEW(new LogField("cached_resp_http_version", "csshv",
+                           LogField::dINT,
+                           &LogAccess::marshal_cache_resp_http_version, &LogAccess::unmarshal_http_version));
+  global_field_list.add(field, false);
+  ink_hash_table_insert(field_symbol_hash, "csshv", field);
+
   field = NEW(new LogField("client_retry_after_time", "crat",
                            LogField::sINT,
                            &LogAccess::marshal_client_retry_after_time, &LogAccess::unmarshal_int_to_str));
@@ -848,34 +903,27 @@ Log::handle_logging_mode_change(const char *name, RecDataT data_type, RecData da
 void
 Log::init(int flags)
 {
-  iObject::Init();
-  iLogBufferBuffer::Init();
-
   maxInactiveObjects = LOG_OBJECT_ARRAY_DELTA;
   numInactiveObjects = 0;
-  inactive_objects = new LogObject *[maxInactiveObjects];
+  inactive_objects = new LogObject*[maxInactiveObjects];
 
   collation_accept_file_descriptor = NO_FD;
 
-  // initialize logging fields
-  //
-  init_fields();
-
   // store the configuration flags
   //
   config_flags = flags;
 
   // create the configuration object
   //
-  config = NEW(new LogConfig);
-  ink_assert(config != NULL);
+  config = NEW (new LogConfig);
+  ink_assert (config != NULL);
 
-  // set the logging_mode and initialize
+  // set the logging_mode and read config variables if needed
   //
   if (config_flags & LOGCAT) {
     logging_mode = LOG_NOTHING;
-  } else {
-
+  }
+  else {
     log_rsb = RecAllocateRawStatBlock((int) log_stat_count);
     LogConfig::register_stat_callbacks();
 
@@ -884,53 +932,66 @@ Log::init(int flags)
 
     if (config_flags & STANDALONE_COLLATOR) {
       logging_mode = LOG_TRANSACTIONS_ONLY;
-      config->collation_mode = LogConfig::COLLATION_HOST;
-    } else {
+    }
+    else {
       int val = (int) LOG_ConfigReadInteger("proxy.config.log.logging_enabled");
-      if (val<LOG_NOTHING || val> FULL_LOGGING) {
+      if (val < LOG_NOTHING || val > FULL_LOGGING) {
         logging_mode = FULL_LOGGING;
-        Warning("proxy.config.log.logging_enabled has an invalid " "value, setting it to %d", logging_mode);
-      } else {
+        Warning("proxy.config.log.logging_enabled has an invalid "
+          "value, setting it to %d", logging_mode);
+      }
+      else {
         logging_mode = (LoggingMode) val;
       }
     }
+  }
 
-    config->init();
-    _init();
+  // if remote management is enabled, do all necessary initialization to
+  // be able to handle a logging mode change
+  //
+  if (!(config_flags & NO_REMOTE_MANAGEMENT)) {
+
+    LOG_RegisterConfigUpdateFunc("proxy.config.log.logging_enabled",
+        &Log::handle_logging_mode_change, NULL);
+
+    LOG_RegisterLocalUpdateFunc("proxy.local.log.collation_mode",
+        &Log::handle_logging_mode_change, NULL);
+
+    // we must create the flush thread since it takes care of the
+    // periodic events (should this behavior be reversed ?)
+    //
+    create_threads();
+
+#ifndef INK_SINGLE_THREADED
+    eventProcessor.schedule_every(NEW (new PeriodicWakeup()), HRTIME_SECOND,
+        ET_CALL);
+#endif
+    init_status |= PERIODIC_WAKEUP_SCHEDULED;
 
     // Clear any stat values that need to be reset on startup
     //
-    LOG_CLEAR_DYN_STAT(log_stat_log_files_open_stat);
-    LOG_CLEAR_DYN_STAT(log_stat_log_files_space_used_stat);
-/*
-        The following variables are not cleared at startup, although
-	we probably should because otherwise their meaning is not very
-	clear. When did we start counting? Does it make sense to have
-	these values since the Traffic Server was setup on the
-	machine?
+    LOG_CLEAR_DYN_STAT( log_stat_log_files_open_stat);
+  }
 
-	LOG_CLEAR_DYN_STAT(log_stat_bytes_written_to_disk_stat);
-	LOG_CLEAR_DYN_STAT(log_stat_bytes_sent_to_network_stat);
-	LOG_CLEAR_DYN_STAT(log_stat_bytes_received_from_network_stat);
-	LOG_CLEAR_DYN_STAT(log_stat_event_log_access_stat);
-	LOG_CLEAR_DYN_STAT(log_stat_event_log_access_skip_stat);
-	LOG_CLEAR_DYN_STAT(log_stat_event_log_access_fail_stat);
-	LOG_CLEAR_DYN_STAT(log_stat_event_log_error_stat);
-*/
-    // if remote management is enabled, do all necessary initialization to
-    // be able to handle a logging mode change
-    //
-    if (!(config_flags & NO_REMOTE_MANAGEMENT)) {
-
-      LOG_RegisterConfigUpdateFunc("proxy.config.log.logging_enabled", &Log::handle_logging_mode_change, NULL);
-      LOG_RegisterLocalUpdateFunc("proxy.local.log.collation_mode", &Log::handle_logging_mode_change, NULL);
+  if (config_flags & LOGCAT) {
+    init_fields();
+  }
+  else {
+    Debug("log-config", "Log::init(): logging_mode = %d "
+        "init status = %d", logging_mode, init_status);
+    init_when_enabled();
+    if (config_flags & STANDALONE_COLLATOR) {
+      config->collation_mode = LogConfig::COLLATION_HOST;
     }
+    config->init();
   }
 }
 
 void
-Log::_init()
+Log::init_when_enabled()
 {
+  init_fields();
+
   if (!(init_status & FULLY_INITIALIZED)) {
 
     if (!(config_flags & STANDALONE_COLLATOR)) {
@@ -1120,36 +1181,31 @@ Log::va_error(char *format, va_list ap)
 void *
 Log::flush_thread_main(void *args)
 {
+  NOWARN_UNUSED (args);
   time_t now, last_time = 0;
-  size_t total_bytes, bytes_to_disk, bytes_to_net, bytes_to_pipe;
+  size_t buffers_flushed;
 
   Debug("log-flush", "Log flush thread is alive ...");
 
   while (true) {
-    ink_timestruc timeout_time;
+    buffers_flushed = 0;
 
-    bytes_to_disk = (bytes_to_net = (bytes_to_pipe = 0));
-    total_bytes = config->log_object_manager.flush_buffers(&bytes_to_disk, &bytes_to_net, &bytes_to_pipe);
+    buffers_flushed = config->log_object_manager.flush_buffers();
 
     if (error_log)
-      total_bytes += error_log->flush_buffers(&bytes_to_disk, &bytes_to_net, &bytes_to_pipe);
+      buffers_flushed += error_log->flush_buffers();
 
-    config->increment_space_used(bytes_to_disk);
+    // config->increment_space_used(bytes_to_disk);
+    // TODO: the bytes_to_disk should be set to Log
 
-    // Update statistics
-    //
-    LOG_SUM_GLOBAL_DYN_STAT(log_stat_bytes_written_to_disk_stat, bytes_to_disk);
-    LOG_SUM_GLOBAL_DYN_STAT(log_stat_bytes_sent_to_network_stat, bytes_to_net);
-
-    Debug("log-flush", "%d bytes flushed this round [ %d to disk, %d to net, %d to pipe]",
-          total_bytes, bytes_to_disk, bytes_to_net, bytes_to_pipe);
+    Debug("log-flush","%zu buffers flushed this round", buffers_flushed);
 
     // Time to work on periodic events??
     //
     now = time(NULL);
     if (now > last_time) {
       if ((now % PERIODIC_TASKS_INTERVAL) == 0) {
-        Debug("log-flush", "periodic tasks for %ld", now);
+        Debug("log-flush", "periodic tasks for %" PRId64, (int64_t)now);
         periodic_tasks(now);
       }
       last_time = (now = time(NULL));
@@ -1158,22 +1214,10 @@ Log::flush_thread_main(void *args)
     // check the queue and find there is nothing to do, then wait
     // again.
     //
-    //ink_cond_wait (&flush_cond, &flush_mutex);
-
-    // vl: we should use ink_cond_timedwait in order to be sure that this thread is alive at least once per second
-    // to execute periodic_tasks()
-    memset(&timeout_time, 0, sizeof(timeout_time));
-    ink_mutex_acquire(&flush_mutex);
-    while (flush_counter < FLUSH_THREAD_MIN_FLUSH_COUNTER && now <= last_time) {
-      timeout_time.tv_sec = (now = time(NULL)) + FLUSH_THREAD_SLEEP_TIMEOUT;
-      if (ink_cond_timedwait(&flush_cond, &flush_mutex, &timeout_time) == ETIMEDOUT)
-        break;
-    }
-    flush_counter = 0;
-    ink_mutex_release(&flush_mutex);
+    ink_cond_wait (&flush_cond, &flush_mutex);
   }
   /* NOTREACHED */
-  return args;
+  return NULL;
 }
 
 /*-------------------------------------------------------------------------
@@ -1261,10 +1305,9 @@ Log::collate_thread_main(void *args)
         continue;
       }
 
-      unsigned version = ntohl(header->version);
-      if (version != LOG_SEGMENT_VERSION) {
-        Note("Invalid LogBuffer received; invalid version - "
-             "buffer = %u, current = %u", version, LOG_SEGMENT_VERSION);
+      if (header->version != LOG_SEGMENT_VERSION) {
+        Note("Invalid LogBuffer received; invalid version - buffer = %u, current = %u",
+             header->version, LOG_SEGMENT_VERSION);
         delete[]header;
         continue;
       }
@@ -1282,14 +1325,6 @@ Log::collate_thread_main(void *args)
       Debug("log-sock", "Using format '%s'", format->name());
 
       delete[]header;
-
-      // vl: absolutely useless code because 'global_buffer_full_list' is not used anywhere
-//          buffer = NEW (new LogBuffer (obj, header));
-//          buffer->convert_to_host_order();
-//          Log::global_buffer_full_list.add (buffer);
-//          ink_mutex_acquire (&flush_mutex);
-//          ink_cond_signal (&Log::flush_cond);
-//          ink_mutex_release (&flush_mutex);
     }
 
     Debug("log", "no longer collation host, deleting LogSock");
